#!/bin/bash
# A GUI tool to analyze the usb drive content
# Misko
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 3 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
PROGNAME="VenToy"
APPNAME="ventoy"
PROGRAMMER="admin"
BUILDDATE="04/22/2020"
VERSION="1.0.06"
WEBSITE="ventoy.net"
EMAIL="<$PROGRAMMER@$WEBSITE>"
COMMANDEXEC="$*"
# set colors so errors etc. stand out.
txtred=$(tput setaf 1)
txtgrn=$(tput setaf 2)
bldblu=$(tput setaf 4)
txtwht=$(tput setaf 7)
BRIGHT=$(tput bold)

# Move to installation folder.
#EXECUTED="${PWD}"
if [[ -d "/usr/share/ventoy/" ]]; then
    cd "/usr/share/ventoy/" || exit 1;
fi
CURRENT="${PWD}"
#cd `dirname $1`
#zenity --info --text="We are in $PWD"
#Reset error count and set as integer.
declare -i ELINE
declare -i ERR
ERR=0
declare -i WARN
WARN=0
declare -i DISTANCE
DISTANCE=0
declare -i INFO
INFO=0
declare -i RR
RR=0
declare -i TT
TT=0
declare -i CALC
CALC=0
declare -i SUCCESS
COLUMNZ=$(tput cols)

# check to see if notify is installed & if sound is supported.
NOTIFY=$(type -p notify-send)
SOUND=$(type -p canberra-gtk-play)

finish () {
    FullBar
}

# trap ctrl-c and call ctrl_c()
ctrl_c () {
        Encapsulate "** Trapped CTRL-C, exiting."
		shopt -s extglob
		finish
		exit 1;
}

trap ctrl_c INT
trap finish EXIT

# Create an array
declare -a ISISO=();

# Debug?
DEBUG=$(echo "$*" | grep -i "debug")

# Set a timer to display duration ##
# Usage:  Timer [start|stop]
Timer () {
	# Debugging switches... Uncomment to see output: 03/30/2016 added as an
	# Optional switch - useful for me and the end user.
    	Tcmd="${1^^}"

    case "$Tcmd" in
        START) _StartSecs=$(date +%s)
            ;;
        STOP) _StopSecs=$(date +%s)
                [[ ! $_StartSecs ]] && Error "[Internal Error] Timer did not record a start time." && return
                ((_DiffSecs=_StopSecs-_StartSecs))
                TimeLapse=$(date -u -d@"$_DiffSecs" +'%-Hh%-Mm%-Ss')
                Encapsulate "Timer: $TimeLapse"
            ;;
        *) Error "[Internal Error] ${FUNCNAME[1]}: Unknown arg '$Tcmd'"
            ;;
    esac
	TTIMEM="$TimeLapse"
}

Timer "Start"

# Download various packages showing progression...
Download() {
	local url="$1"
	echo -n "    "
	wget --progress=dot "$url" 2>&1 | grep --line-buffered "%" | \
	sed -u -e "s,\.,,g" | awk '{printf("\b\b\b\b%4s", $2)}'
	echo -ne "\b\b\b\b"
	Encapsulate "Done."
}

# Dump data to the screen in pretty columns.
Columnize () {
	# Columnize is an internally called function exclusively.
	# EXAMPLE
	# -t to shove through the column titles we will count the number of switches and
	# calculate spacing eventually.
	if [[ "$1" == "-t" ]]; then
		#Encapsulate "${bldgrn}Number of switches passed: $#"
		for ARG in "$@"
		do
			array_counter=$((array_counter + 1))
			if [[ "$array_counter" -gt 2 ]]; then
				#echo "Switch $array_counter: $ARG"
				ARGUMENTS[$array_counter]="$ARG"
				tLen="${#ARGUMENTS[$array_counter]}"
				if [[ $tLen -ge $WIDEST ]]; then
					WIDEST="$tLen"
				fi
				#echo "Length of ${ARGUMENTS[$array_counter]} is $tLen"
			fi
			#echo "The widest passed varible is $WIDEST"
		done
	else
		#Encapsulate "${bldgrn}Number of switches passed: $#"
		for ARG in "$@"
		do
			array_counter=$((array_counter + 1))
			if [[ "$array_counter" -gt 1 ]]; then
				#echo "Switch $array_counter: $ARG"
				ARGUMENTS[$array_counter]="$ARG"
				tLen="${#ARGUMENTS[$array_counter]}"
				if [[ "$tLen" -ge "$WIDEST" ]]; then
					WIDEST="$tLen"
				fi
				#echo "Length of ${ARGUMENTS[$array_counter]} is $tLen"
			fi
			#echo "The widest passed varible is $WIDEST"
		done
	fi
	#echo "#: $# VARS: $*"
	#LONGESTLENGTH="${#2}"
	#echo "Length of $2 is $LONGESTLENGTH"
	#echo $COLUMNZ
	MAINS=$(echo "scale=2; $COLUMNZ-30" | bc)
	MAINS=${MAINS%.*}
	#echo $MAINS
	MAINS=$((MAINS - 2))
	MAINS=$((MAINS / 4))
	#echo $MAINS
	if [[ "$1" = '-t' ]]; then
		#"#" "DEV" "VENDOR" "LABEL" "TYPE" "SIZE"
		printf "%-0s %-70s %-11s %-16s %-7s %-3s %-11s %-3s %-14s %-1s\n" "▒" "$2" "$3" "$4" "$5" "$6" "$7" "$8"
	else
		#MAINS=$(expr $MAINS / 2)
		printf "%-0s %-70s %-11s %-16s %-7s %-3s %-11s %-3s %-14s %-1s\n" "▒" "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8"
	fi
}

# for outputting to X-11 screen a notification of completion and playing sound.
# The sound file is from the "Optional" ultimate-edition-sound-theme
Notification() {
	# Is X11 running?
	X=$( pidof X )
	# Does the user have the optional UE Sound scheme installed?
	if [[ -f "/usr/share/sounds/Ultimate Edition Sound Scheme/stereo/itisdone.wav" ]]; then
		WAVE=1
	fi
	# Is the user in an X11 GUI environment and have notification OSD installed?
	if [[ "$X" && "$NOTIFY" ]]; then
		notify-send "${APPNAME}" "$1" -i /usr/share/ultimate_edition/logo.png -t 5000
	fi
	# Does the user have pre-requisites? If so, break out the Cylon voice.
	if [[ "$SOUND" && "$WAVE" ]]; then
		canberra-gtk-play -i itisdone 2>/dev/null
	fi
	# Display message in terminal unconditionally
	Encapsulate "$1"
}

# Usage: Eye Candy "Message"
# Example: Eye Candy "Repostorm $REPOVERSION is entering extraction mode."
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ Repostorm 1.7.7 is entering extraction mode. ▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
EyeCandy() {
	# Verify a message has been sent to this otherwise do squat.

	STP="$1"
	if [[ "$STP" ]]; then
		STRLEN="${#STP}"
		((STRLEN=STRLEN + 3))
		GREPPED=$(echo "$STP" | grep -E '\e')
		if [[ "$GREPPED" ]]; then
			((STRLEN=STRLEN - 13))
		fi
		RR=0; while [ "$RR" -le "$STRLEN" ]; do echo -n "▒"; ((RR=RR+1)); done; echo;
		Encapsulate "${bldblu}▒${txtgrn} $STP ${bldblu}▒"
		RR=0; while [ "$RR" -le "$STRLEN" ]; do echo -n "▒"; ((RR=RR+1)); done; echo;
	fi
}

# Usage: Encapsulate
# Encapsulate "Repostorm 1.8.7 is entering extraction mode."
# Example: ▒ Repostorm 1.8.7 is entering extraction mode. ▒
Encapsulate() {
	# Verify a message has been sent to this otherwise do squat.
	# Example: Encapsulate ('Repostorm $REPOVERSION is entering extraction mode.')
	# ▒ Repostorm 1.7.7 is entering extraction mode.                       ▒

	STP="$1"
	if [[ "$STP" ]]; then
		STRLEN="${#STP}"
		((STRLEN=STRLEN + 3))
		((DISTANCE=COLUMNZ - STRLEN))
		GREPPED=$(echo "$STP" | grep -E '\e')
		if [[ "$GREPPED" ]]; then
			((STRLEN=STRLEN - 13))
		fi
		echo -e -n "${BRIGHT}${bldblu}▒ ${BRIGHT}${txtgrn}$STP"
		RR=0; while [ "$RR" -lt "$DISTANCE" ]; do echo -n " "; ((RR=RR+1)); done; echo "${BRIGHT}${bldblu}▒";
	fi
}

# Center data passed to on screen
Center() {
	STP="$1"
	ERROR="$2"
	if [[ "$STP" ]]; then
		STRLEN="${#STP}"
		((CENTER=STRLEN / 2 ))
		#CENTER=CENTER-1
		((TCENTER=COLUMNZ / 2 ))
		#TCENTER=TCENTER-1
		((DISTANCE=TCENTER - CENTER))
		((DISTANCE=DISTANCE - 2))
		if ! [[ "$ERROR" ]]; then
			echo -e -n "${txtrst}"
		else
			echo -e -n "${txtred}"
		fi
		RR=0; while [ "$RR" -lt "$DISTANCE" ]; do echo -e -n "${BRIGHT}${bldblu}▒"; ((RR=RR+1)); done;
		echo -e -n "${BRIGHT}${txtgrn} $STP "
		if ! [[ "$ERROR" ]]; then
			echo -e -n "${txtrst}"
		else
			echo -e -n "${txtred}"
		fi
		TT=0; while [ "$TT" -lt "$DISTANCE" ]; do echo -e -n "${BRIGHT}${bldblu}▒"; ((TT=TT+1)); done;
		((CALC=RR + TT + STRLEN + 2))
		if [[ "$CALC" -lt "$COLUMNZ" ]]; then
			while [ "$CALC" -lt "$COLUMNZ" ]; do echo -e -n "${BRIGHT}${bldblu}▒"; ((CALC=CALC+1)); done; echo;
		fi
	fi
	# echo "STRING LEN: $STRLEN CENTER: $CENTER TOTAL CNTER: $TCENTER RR:$RR TT:$TT COLUMNZ: $COLUMNZ CALC: $CALC DISTANCE: $DISTANCE"
	echo -e -n "${txtrst}"
}

Error() {
	echo -e -n "${txtred}"
	#FullBar
	Center "$1" "--ERROR"
	#echo -e -n "${txtred}"
	#FullBar
	echo -e -n "${txtrst}"
}

# Draws a bar across the screen based on screen size detected (columnz)
FullBar() {
	RR=1; while [ "$RR" -le "$COLUMNZ" ]; do echo -e -n "${BRIGHT}${bldblu}▒"; ((RR=RR+1)); done; echo;
}

# Dump version information and exit function
VersionDump() {
	# Debugging switches... Uncomment to see output: 03/30/2016 added as an
	# Optional switch - useful for me and the end user.
	AMOUNT=0
	if [[ "$DEBUG" ]]; then
		Center "DEBUG INFORMATION: ${FUNCNAME[1]}"
		for EACH in "$@"
		do
			((AMOUNT ++))
			echo "$AMOUNT:${EACH} "
		done
		echo -e -n "Caller [LINE] / APP: "
		caller
		FullBar
		echo -e -n "${txtrst}"
	fi

	GREPIT=$(echo "$*" | grep "suppress")
	if ! [[ "$GREPIT" ]]; then
		Center "${APPNAME} (${PROGNAME}) ${VERSION}, $BUILDDATE"
		Encapsulate "GNU ${PROGNAME} home page: <http://$WEBSITE/>."
		Encapsulate "E-mail bug reports to: $EMAIL."
		Encapsulate "Be sure to include the word ${PROGNAME} somewhere in the Subject: field."
		FullBar
	else
		echo "${APPNAME} (${PROGNAME}) ${VERSION}, $BUILDDATE"
		echo "GNU ${PROGNAME} home page: <http://$WEBSITE/>."
		echo "E-mail bug reports to: $EMAIL."
		echo "Be sure to include the word ${PROGNAME} somewhere in the Subject: field."
	fi
}

# Make temporary file
LIST=$(mktemp /tmp/XXXXXX)
VersionDump
declare -a DEVS=();
declare -a DEVICES=();
# get a list of DEVices
if [[ -d "/dev/disk/by-path/" ]]; then
    cd "/dev/disk/by-path/" || exit 1;
    DEVS=(*usb*part*)
    SAVEIFS=$IFS
    IFS=$'\n'
    DEVICES=$(readlink -e "${DEVS[@]}")
    DEVICES=(${DEVICES})
    IFS="$SAVEIFS"
    # Change IFS to new line.
    for EACH in "${DEVICES[@]}"
    do
        #SAVEIFS=$IFS
        # Change IFS to new line.
        #IFS=$'\n'
        #DEVICES+=$(readlink -e "${EACH}")
        #DEVICES=(${DEVICES})
        #IFS="$SAVEIFS"
        Encapsulate "DEVICE: $EACH"
    done
fi
COUNT=0
for EACH in "${DEVS[@]}"
do
    Encapsulate "FOUND DEVICE: #$COUNT ${EACH} DEVICE: ${DEVICES[$COUNT]}"
    COUNT=$((COUNT +1))
done
FullBar
#DEVs=$(ls -al /dev/disk/by-path/*usb*part* 2>/dev/null | awk '{print($11)}')
if [[ "${?}" != 0 || -z "${DEVS}" ]]; then
    zenity --warning --text "No USB Mounted!"
    exit 0;
fi

# Initialize list and make sure it is empty
>"${LIST}"
BLANK="FALSE"
COUNT=0
# Now get the info about our DEVices and put it in a list

for DEV in "${DEVICES[@]}"
do
    echo "FALSE" >> "${LIST}"
    DEVICE="${DEV##*\/}";
    echo -n "$DEVICE" >> "${LIST}"
    LETTER="${DEVICE//[[:digit:]]/}"
    #echo -n "$DEV" >> "${LIST}"
    echo " " >> "${LIST}"
    VENDOR=$(cat /sys/block/${DEVICE:0:3}/device/vendor 2>/dev/null)
    echo "${VENDOR}" >> "${LIST}"
    MODEL=$(cat /sys/block/${DEVICE:0:3}/device/model 2>/dev/null)
    echo "${MODEL}" >> "${LIST}"
    DISKSIZE=$(lsblk 2>/dev/null | grep -E "${DEVICE}" | awk '{print($4)}')
    echo "${DISKSIZE}" >> "${LIST}"
    MOUNTPOINT=$(lsblk -o NAME,TRAN,MOUNTPOINT | grep "${LETTER}" | grep -A 1 -w usb | grep -v usb | awk '{print $2}')
    echo "${MOUNTPOINT}" >> "${LIST}"
    #echo -n `lsblk -o NAME 2>/dev/null | grep \`echo -E ${dev}\`| awk '{print($2)}'  ` >> $list;
    FSTYPE=$(df -T /dev/${DEVICE} 2>/dev/null | tail -1 | awk '{print $2}')
    echo "${FSTYPE}" >> "${LIST}"
    #df -T `echo -E /dev/${dev}` 2>/dev/null | tail -1 | awk '{print $2}' >> $list;
    UUID=$(ls -l /dev/disk/by-uuid/ 2>/dev/null | grep -E "${DEVICE}" | awk '{print($9)}')
    echo "${UUID}" >> "${LIST}"
    #echo `ls -l /dev/disk/by-uuid/ 2>/dev/null | grep \`echo -E ${dev}\`| awk '{print($9)}' ` >> $list;
done


# clear our array
DEVfs=()

# read in the array using a line tmp variable. This let's zenity see it line by line
while read -r line
do
    DEVfs+=("${line}")
done < "${LIST}"

rm "${LIST}"
unset list

#Display the main dialog
USBDISK=$(zenity --list --title "$PROGNAME" --window-icon="/usr/share/ventoy/ventoy/ventoy.png" --text="Select your USB drive from the list and click OK to begin.\nWARNING: This will destroy the selected drive!!!" --radiolist  --width=670 --height=350 --column="Pick" --column="Dev" --column="Vendor" --column="Model" --column="Size" --column="Mount" --column="FS" --column="UUID" "${DEVfs[@]}" --separator=":")

#Test for cancellation
if [[ "${?}" != 0  ]]
then
    exit 0;
fi

#Check if anything was selected
echo "${USBDISK}" | grep ^[0-9a-zA-Z] &>/dev/null
if [[ "${?}" != 0  ]]; then
    zenity --info --title "$PROGNAME" --window-icon="/usr/share/ventoy/ventoy/ventoy.png" --text="Nothing was selected..." --width=150 --height=100 --timeout=10
    exit 0;
fi
ACTUALDRIVE="${USBDISK//[[:digit:]]/}"
ANSWER=$(zenity --question --title "$PROGNAME" --icon-name="dialog-warning" --window-icon="/usr/share/ventoy/ventoy/ventoy.png" --text="Selected drive is ${USBDISK}, this ENTIRE drive: ${ACTUALDRIVE} will be destroyed ${ACTUALDRIVE}2, ${ACTUALDRIVE}3, etc.\nAre you sure?" --width=350 --height=180)
if [[ "${?}" != 0  ]]; then
    exit 0;
else
    Encapsulate "Unmounting: ${ACTUALDRIVE}"
    UNMOUNT=$(umount "/dev/${ACTUALDRIVE}" 2>/dev/null) 
    for I in {1..4}
    do
        UNMOUNT=$(umount "/dev/${ACTUALDRIVE}${I}" 2>/dev/null) 
    done    
fi
if [[ -d "${CURRENT}" ]]; then
    cd "${CURRENT}" || exit 1;
    LASTCHANCE=$(zenity --question --title "$PROGNAME" --text="LAST CHANCE BEFORE DRIVE IS DESTROYED!!!\nExecute: Ventoy2Disk.sh -I  /dev/${ACTUALDRIVE} in $PWD" --window-icon="/usr/share/ventoy/ventoy/ventoy.png" --width=350 --height=180)
    if [[ "${?}" != 0  ]]; then
        exit 0;
    else
    Encapsulate "Executing: Ventoy2Disk.sh -I /dev/${ACTUALDRIVE} in $PWD"
    yes | sudo ./Ventoy2Disk.sh -I "/dev/${ACTUALDRIVE}" | zenity --progress --window-icon="/usr/share/ventoy/ventoy/ventoy.png" --width 350 --pulsate --text "Installing $PROGNAME on device: ${ACTUALDRIVE}, please wait..." --title "$PROGNAME"
    fi
fi
Notification "$PROGNAME, all operations complete."
exit 0;
